﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse;
using RimWorld;
using Verse.AI.Group;

namespace RaiderInfo {
    public class Alert_RaiderLeft : Alert {
        private IEnumerable<Lord> RaiderLords {
            get {
                List<Map> maps = Find.Maps;
                for (int i = 0; i < maps.Count; i++) {
                    if (maps[i].IsPlayerHome) {
                        foreach (Lord lord in maps[i].lordManager.lords) {
                            if (lord.faction != null && lord.ownedPawns != null) {
                                if (lord.faction.def == FactionDefOf.Insect && !RaiderInfoMod.Settings.alertForInsects) {
                                    continue;
                                }
                                if (lord.faction.HostileTo(Faction.OfPlayer) && lord.ownedPawns.All((p) => { return p.mindState.Active; }) && lord.ownedPawns.Exists(p => p.Map != null && !p.Map.fogGrid.IsFogged(p.Position))) {
                                    yield return lord;
                                }
                            }
                        }
                    }
                }
            }
        }

        public Alert_RaiderLeft() {
            this.defaultLabel = "RaiderInfo.RaiderLeftLabel".Translate();
            this.defaultPriority = AlertPriority.High;
        }

        public override string GetExplanation() {
            StringBuilder stringBuilder = new StringBuilder();
            
            if (this.RaiderLords != null) {
                foreach (Lord lord in this.RaiderLords) {
                    if (stringBuilder.Length > 0) {
                        stringBuilder.AppendLine();
                    }
                    stringBuilder.Append(GetRaiderLeft(lord));
                }
            }

            return stringBuilder.ToString();
        }

        public string GetRaiderLeft(Lord lord) {
            string factionName = lord.faction.Name;
            int raiderLeft = GetPawns(lord);
            bool isHumanLike = lord.faction != null && lord.faction.leader != null && lord.faction.leader.RaceProps != null && lord.faction.leader.RaceProps.Humanlike;
            string unitOfRaider = isHumanLike ? "RaiderInfo.RaiderLeftUnitOfHuman".Translate() : "RaiderInfo.RaiderLeftUnitOfNonHuman".Translate();

            return "RaiderInfo.RaiderLeft".Translate( factionName, raiderLeft, unitOfRaider );
        }

        public int GetPawns(Lord lord) {
            int count = 0;

            foreach (Pawn p in lord.ownedPawns) {
                if (p.Spawned && !p.Downed && !p.Dead && !p.Map.fogGrid.IsFogged(p.Position)) {
                    //Log.Message(p.Label + "/" + p.Map.fogGrid.IsFogged(p.Position));
                    count++;
                }
            }

            return count;
        }

        public override AlertReport GetReport() {
            if (Find.AnyPlayerHomeMap == null) {
                return false;
            }

            Lord lord = this.RaiderLords.FirstOrDefault<Lord>();
            if (lord == null) {
                return false;
            }

            List<Pawn> list = new List<Pawn>();
            foreach (Lord l in this.RaiderLords) {
                if (l.ownedPawns != null) {
                    foreach (Pawn p in l.ownedPawns.FindAll(p => p.Map != null && !p.Map.fogGrid.IsFogged(p.Position))) {
                        list.Add(p);
                    }
                }
            }

            int index = Patch_CameraJumper_TryJumpAndSelect.count % list.Count;
            return list[index];
        }
    }
}
