﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse;
using RimWorld;
using Verse.AI.Group;
using HarmonyLib;

namespace RaiderInfo {
    public class Alert_MechAssembler : Alert {
        private IEnumerable<Thing> MechAssemblers {
            get {
                if (!ModsConfig.RoyaltyActive) {
                    yield break;
                }
                List<Map> maps = Find.Maps;
                for (int i = 0; i < maps.Count; i++) {
                    Map map = maps[i];
                    foreach (Thing t in map.listerThings.ThingsOfDef(ThingDef.Named("MechAssembler"))) {
                        yield return t;
                    }
                }
            }
        }

        private List<Thing> MechAssemblersOrderByNextSpawnTick {
            get {
                if (this.MechAssemblers == null) {
                    return null;
                }
                Func<Thing, int> selector = delegate (Thing t) {
                    CompSpawnerPawn comp = t.TryGetComp<CompSpawnerPawn>();
                    if (comp != null) {
                        return comp.nextPawnSpawnTick;
                    }
                    return -1;
                };
                return this.MechAssemblers.OrderBy(selector).ToList();
            }
        }

        public Alert_MechAssembler() {
            this.defaultPriority = AlertPriority.High;
        }

        public override string GetLabel() {
            List<Thing> list = this.MechAssemblersOrderByNextSpawnTick;
            if (!list.NullOrEmpty()) {
                CompSpawnerPawn comp = list.First().TryGetComp<CompSpawnerPawn>();
                int leftTick = comp.nextPawnSpawnTick - Find.TickManager.TicksGame;
                return "RaiderInfo.MechAssemblerLabel".Translate(leftTick.ToStringTicksToPeriod());
            }
            return "";
        }

        public override TaggedString GetExplanation() {
            StringBuilder stringBuilder = new StringBuilder();
            if (this.MechAssemblers != null) {
                foreach (Thing mechAssembler in this.MechAssemblersOrderByNextSpawnTick) {
                    if (stringBuilder.Length > 0) {
                        stringBuilder.AppendLine();
                    }
                    CompSpawnerPawn comp = mechAssembler.TryGetComp<CompSpawnerPawn>();
                    if (comp != null) {
                        PawnKindDef chosenKind = Traverse.Create(comp).Field("chosenKind").GetValue<PawnKindDef>();
                        int leftTick = comp.nextPawnSpawnTick - Find.TickManager.TicksGame;
                        stringBuilder.Append("RaiderInfo.MechAssemblerDescItem".Translate(chosenKind.LabelCap, leftTick.ToStringTicksToPeriod()));
                    }
                }
            }
            return "RaiderInfo.MechAssemblerDescBase".Translate(stringBuilder.ToString());
        }

        public override AlertReport GetReport() {
            if (!ModsConfig.RoyaltyActive) {
                return false;
            }

            if (Find.AnyPlayerHomeMap == null) {
                return false;
            }

            if (this.MechAssemblers == null) {
                return false;
            }

            return AlertReport.CulpritsAre(this.MechAssemblersOrderByNextSpawnTick);
        }
    }
}
